@@ -12,6 +12,7 @@ from .const import SITE_PACKAGES_PATHS
 from .const import SYS_PREFIX_PATHS
 from .util import CYTHON_SUFFIX_RE
 from .util import LEADING_WHITESPACE_RE
+from .util import MISSING
 from .util import cached_property
 from .util import get_func_in_mro
 from .util import get_main_thread
@@ -39,8 +40,21 @@ class Event(object):
     arg = None
     depth = None
     calls = None
+    builtin = None
+
+    def __init__(self, frame, kind, arg, tracer=None, depth=None, calls=None, threading_support=MISSING):
+        if tracer is None:
+            if depth is None:
+                raise TypeError('Missing argument: depth (required because tracer was not given).')
+            if calls is None:
+                raise TypeError('Missing argument: calls (required because tracer was not given).')
+            if threading_support is MISSING:
+                raise TypeError('Missing argument: threading_support (required because tracer was not given).')
+        else:
+            depth = tracer.depth
+            calls = tracer.calls
+            threading_support = tracer.threading_support
 
-    def __init__(self, frame, kind, arg, tracer):
         #: The original Frame object.
         #:
         #: .. note::
@@ -49,8 +63,19 @@ class Event(object):
         #:  You may access it from your custom predicate though.
         self.frame = frame
 
-        #: The kind of the event, could be one of ``'call'``, ``'line'``, ``'return'``, ``'exception'``,
-        #: ``'c_call'``, ``'c_return'``, or ``'c_exception'``.
+        if kind.startswith('c_'):
+            kind = kind[2:]
+            builtin = arg
+            arg = None
+        else:
+            builtin = False
+
+        #: If kind of the event is one of ``'c_call'``, ``'c_return'``, or ``'c_exception'`` then this will be True.
+        #:
+        #: :type: builtin function or False
+        self.builtin = builtin
+
+        #: The kind of the event, could be one of ``'call'``, ``'line'``, ``'return'``, ``'exception'``.
         #:
         #: :type: str
         self.kind = kind
@@ -58,15 +83,15 @@ class Event(object):
         #: A value that depends on ``kind``
         self.arg = arg
 
-        #: Tracing depth (increases on calls, decreases on returns)
+        #: Tracing depth (increases on calls, decreases on returns).
         #:
         #: :type: int
-        self.depth = tracer.depth
+        self.depth = depth
 
         #: A counter for total number of calls up to this Event.
         #:
         #: :type: int
-        self.calls = tracer.calls
+        self.calls = calls
 
         #: A copy of the :attr:`hunter.tracer.Tracer.threading_support` flag.
         #:
@@ -75,7 +100,7 @@ class Event(object):
         #:  Not allowed in the builtin predicates. You may access it from your custom predicate though.
         #:
         #: :type: bool or None
-        self.threading_support = tracer.threading_support
+        self.threading_support = threading_support
 
         #: Flag that is ``True`` if the event was created with :meth:`~hunter.event.Event.detach`.
         #:
@@ -125,6 +150,7 @@ class Event(object):
         event.__dict__['stdlib'] = self.stdlib
         event.__dict__['threadid'] = self.threadid
         event.__dict__['threadname'] = self.threadname
+        event.__dict__['instruction'] = self.instruction
 
         if value_filter:
             event.__dict__['arg'] = value_filter(self.arg)
@@ -139,6 +165,7 @@ class Event(object):
         event.calls = self.calls
         event.depth = self.depth
         event.kind = self.kind
+        event.builtin = self.builtin
 
         event.detached = True
 
@@ -149,6 +176,17 @@ class Event(object):
         event.__dict__ = dict(self.__dict__)
         return event
 
+    @cached_property
+    def instruction(self):
+        """
+        Last byte instruction. If no bytecode was used (Cython code) then it returns ``None``.
+        Depending on Python version it might be an int or a single char string.
+
+        :type: int or single char string or None
+        """
+        if self.frame.f_lasti >= 0 and self.frame.f_code.co_code:
+            return self.frame.f_code.co_code[self.frame.f_lasti]
+
     @cached_property
     def threadid(self):
         """
@@ -183,6 +221,9 @@ class Event(object):
 
         :type: dict
         """
+        if self.builtin:
+            return {}
+
         return self.frame.f_locals
 
     @cached_property
@@ -192,6 +233,9 @@ class Event(object):
 
         :type: dict
         """
+        if self.builtin:
+            return {}
+
         return self.frame.f_globals
 
     @cached_property
@@ -201,7 +245,10 @@ class Event(object):
 
         :type: str
         """
-        return self.code.co_name
+        if self.builtin:
+            return self.builtin.__name__
+        else:
+            return self.code.co_name
 
     @cached_property
     def function_object(self):
@@ -217,6 +264,9 @@ class Event(object):
         :type: function or None
         """
         # Based on MonkeyType's get_func
+        if self.builtin:
+            return self.builtin
+
         code = self.code
         if code.co_name is None:
             return None
@@ -248,10 +298,12 @@ class Event(object):
 
         :type: str
         """
+        if self.builtin:
+            return self.builtin.__module__
+
         module = self.frame.f_globals.get('__name__', '')
         if module is None:
             module = ''
-
         return module
 
     @cached_property
@@ -262,6 +314,10 @@ class Event(object):
 
         :type: str
         """
+        # if self.builtin:
+        #     return '<builtin>'
+        # if self.builtin:
+        #     return '<builtin>'
         filename = self.frame.f_code.co_filename
         if not filename:
             filename = self.frame.f_globals.get('__file__')
@@ -371,7 +427,7 @@ def yield_lines(filename, module_globals, start, collector,
     for line in linecache.getlines(filename, module_globals)[start:start + limit]:
         if dedent is None:
             dedent = leading_whitespace_re.findall(line)
-            dedent = dedent[0] if dedent else ""
+            dedent = dedent[0] if dedent else ''
             amount = len(dedent)
         elif not line.startswith(dedent):
             break
